from typing import Optional

from abqpy.decorators import abaqus_function_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant

"""The Python module textRepr contains functions to print an Abaqus object and to convert 
the output from the str() function into a formatted string, indented for each object in 
the recursive listing of a Python object. 


.. note::
    Theses functions can be accessed by::

        import textRepr

"""


@abaqus_function_doc
def getIndentedRepr(
    object,
    maxRecursionDepth: Optional[SymbolicConstant] = None,
    maxElementsInSequence: Optional[SymbolicConstant] = None,
    significantDigits: float = 6,
):
    """This method returns a String with each level of parentheses indented.

    Parameters
    ----------
    object
        A Python object to be processed. (The Python object can be an Abaqus object.) This
        argument can also be a String representation of a Python object obtained from
        str(*object*).
    maxRecursionDepth
        An Int specifying the maximum depth to which the object will be printed, the
        SymbolicConstant UNLIMITED, or None. A value of None implies that the current setting in
        the TextReprOptions object will be used. If **object** is not an Abaqus object,
        **maxRecursionDepth** has no effect. The default value is None.You should take care when
        setting **maxRecursionDepth** = UNLIMITED because the resulting output can be very large. To
        limit the output, you should set **maxElementsInSequence** to a small number.
    maxElementsInSequence
        An Int specifying the maximum number of elements of a sequence to return or the
        SymbolicConstant UNLIMITED. The initial value is 100. After the maximum number of
        elements have been printed, the remainder are indicated by the string '...'.
    significantDigits
        An Int specifying the number of significant digits for Floats in the output. Possible
        values are 0 < **significantDigits** ≤ 15. The default value is 6.

    Returns
    -------
    str
        A String.
    """
    ...


@abaqus_function_doc
def getPaths(
    object,
    maxRecursionDepth: Optional[SymbolicConstant] = None,
    maxElementsInSequence: Optional[SymbolicConstant] = None,
    pathRoot: str = "",
):
    """This method processes the argument and interprets its structure. It then returns a
    String containing the object paths of all the child objects found.

    Parameters
    ----------
    object
        A Python object to be processed. (The Python object can be an Abaqus object.) This
        argument can also be a String representation of a Python object.
    maxRecursionDepth
        An Int specifying the maximum depth to which the object will be printed, the
        SymbolicConstant UNLIMITED, or None. A value of None implies that the current setting in
        the TextReprOptions object will be used. If **object** is not an Abaqus object,
        **maxRecursionDepth** has no effect. The default value is None.You should take care when
        setting **maxRecursionDepth** = UNLIMITED because the resulting output can be very large. To
        limit the output, you should set **maxElementsInSequence** to a small number.
    maxElementsInSequence
        An Int specifying the maximum number of elements of a sequence to return or the
        SymbolicConstant UNLIMITED. The initial value is 100. After the maximum number of
        elements have been printed, the remainder are indicated by the string '...'.
    pathRoot
        A String specifying the root of the paths. This String is prepended to each path found.
        The default value is None, implying that the path to **object** will be used unless
        **object** is a String.

    Returns
    -------
    str
        A String.
    """
    ...


@abaqus_function_doc
def getTypes(
    object,
    maxRecursionDepth: Optional[SymbolicConstant] = None,
    maxElementsInSequence: Optional[SymbolicConstant] = None,
    pathRoot: str = "",
):
    """This method processes the argument, interprets its structure, and returns a String
    containing all the object types within the object in the form '*TypeName* **Path**'.

    Parameters
    ----------
    object
        An Abaqus object.
    maxRecursionDepth
        An Int specifying the maximum depth to which the object will be printed, the
        SymbolicConstant UNLIMITED, or None. A value of None implies that the current setting in
        the TextReprOptions object will be used. The default value is None.You should take care
        when setting **maxRecursionDepth** = UNLIMITED because the resulting output can be very
        large. To limit the output, you should set **maxElementsInSequence** to a small number.
    maxElementsInSequence
        An Int specifying the maximum number of elements of a sequence to return or the
        SymbolicConstant UNLIMITED. The initial value is 100. After the maximum number of
        elements have been printed, the remainder are indicated by the string '...'.
    pathRoot
        A String specifying the root of the paths. This String is prepended to each path found.
        The default value is None, implying that the path to **object** will be used unless
        **object** is a String.

    Returns
    -------
    str
        A String.
    """
    ...


@abaqus_function_doc
def prettyPrint(
    object,
    maxRecursionDepth: Optional[SymbolicConstant] = None,
    maxElementsInSequence: Optional[SymbolicConstant] = None,
    significantDigits: float = 6,
):
    """This method prints a formatted version of the object. The prettyPrint function uses
    getIndentedRepr to format the String representation, but does not save the full text
    output of getIndentedRepr. Therefore this function can be used in cases where
    getIndentedRepr would run out of memory.

    Parameters
    ----------
    object
        An Abaqus object.
    maxRecursionDepth
        An Int specifying the maximum depth to which the object will be printed, the
        SymbolicConstant UNLIMITED, or None. A value of None implies that the current setting in
        the TextReprOptions object will be used. The default value is None.You should take care
        when setting **maxRecursionDepth** = UNLIMITED because the resulting output can be very
        large. To limit the output, you should set **maxElementsInSequence** to a small number.
    maxElementsInSequence
        An Int specifying the maximum number of elements of a sequence to return or the
        SymbolicConstant UNLIMITED. The initial value is 100. After the maximum number of
        elements have been printed, the remainder are indicated by the string '...'.
    significantDigits
        An Int specifying the number of significant digits for Floats in the output. Possible
        values are 0 < **significantDigits** ≤ 15. The default value is 6.
    """
    ...


@abaqus_function_doc
def printPaths(
    object,
    maxRecursionDepth: Optional[SymbolicConstant] = None,
    maxElementsInSequence: Optional[SymbolicConstant] = None,
    pathRoot: str = "",
):
    """This method prints the object path of the **object** argument and its members, depending
    on the **maxRecursionDepth** argument. The printPaths function uses getPaths to print a
    list of paths to each of the child objects in **object**.

    Parameters
    ----------
    object
        An Abaqus object. This argument can also be a String representation of an Abaqus object
        obtained from str(*object*).
    maxRecursionDepth
        An Int specifying the maximum depth to which the object will be printed, the
        SymbolicConstant UNLIMITED, or None. A value of None implies that the current setting in
        the TextReprOptions object will be used. The default value is None.You should take care
        when setting **maxRecursionDepth** = UNLIMITED because the resulting output can be very
        large. To limit the output, you should set **maxElementsInSequence** to a small number.
    maxElementsInSequence
        An Int specifying the maximum number of elements of a sequence to return or the
        SymbolicConstant UNLIMITED. The initial value is 100. After the maximum number of
        elements have been printed, the remainder are indicated by the string '...'.
    pathRoot
        A String specifying the root of the paths to be printed. This String is prepended to
        each path found. The default value is None, implying that the path to **object** will be
        used.
    """
    ...


@abaqus_function_doc
def printTypes(
    object,
    maxRecursionDepth: Optional[SymbolicConstant] = None,
    maxElementsInSequence: Optional[SymbolicConstant] = None,
    pathRoot: str = "",
):
    """This method prints the object type. The printTypes function uses getTypes to print a
    list of all the object types in **object**.

    Parameters
    ----------
    object
        An Abaqus object.
    maxRecursionDepth
        An Int specifying the maximum depth to which the object will be printed, the
        SymbolicConstant UNLIMITED, or None. A value of None implies that the current setting in
        the TextReprOptions object will be used. The default value is None.You should take care
        when setting **maxRecursionDepth** = UNLIMITED because the resulting output can be very
        large. To limit the output, you should set **maxElementsInSequence** to a small number.
    maxElementsInSequence
        An Int specifying the maximum number of elements of a sequence to return or the
        SymbolicConstant UNLIMITED. The initial value is 100. After the maximum number of
        elements have been printed, the remainder are indicated by the string '...'.
    pathRoot
        A String specifying the root of the paths to be printed. This String is prepended to
        each path found. The default value is None, implying that the path to **object** will be
        used.
    """
    ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .ButterworthFilter import ButterworthFilter
from .Chebyshev1Filter import Chebyshev1Filter
from .Chebyshev2Filter import Chebyshev2Filter
from .OperatorFilter import OperatorFilter
from ..Odb.OdbBase import OdbBase
from ..UtilityAndView.abaqusConstants import Boolean, NONE, OFF
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class FilterOdb(OdbBase):
    """The Odb object is the in-memory representation of an output database (ODB) file.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name]
    """

    @abaqus_method_doc
    def ButterworthFilter(
        self,
        name: str,
        cutoffFrequency: float,
        order: int = 2,
        operation: Literal[C.MIN, C.MAX, C.NONE, C.ABS] = NONE,
        halt: Boolean = OFF,
        limit: Optional[float] = None,
        invariant: Literal[C.FIRST, C.SECOND, C.NONE] = NONE,
    ) -> ButterworthFilter:
        """This method creates a ButterworthFilter object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ButterworthFilter
                session.odbs[name].ButterworthFilter

        Parameters
        ----------
        name
            A String specifying the repository key. This name ANTIALIASING is reserved for filters
            generated internally by the program.
        cutoffFrequency
            A Float specifying the attenuation point of the filter. Possible values are non-negative
            numbers. Order is not available for OperatorFilter.
        order
            An Int specifying the highest power of the filter transfer function. Possible values are
            non-negative numbers less than or equal to 20. Order is not available for
            OperatorFilter. The default value is 2.
        operation
            A SymbolicConstant specifying the filter operator. Possible values are NONE, MIN, MAX,
            and ABS. The default value is NONE.
        halt
            A Boolean specifying whether to stop the analysis if the specified limit is reached. The
            default value is OFF.
        limit
            None or a Float specifying the threshold limit, an upper or lower bound for output
            values depending on the operation, or a bound for stopping the analysis when Halt is
            used. The default value is None.
        invariant
            A SymbolicConstant specifying the invariant to which filtering is applied. Possible
            values are NONE, FIRST, and SECOND. The default value is NONE.

        Returns
        -------
        ButterworthFilter
            A ButterworthFilter object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        self.filters[name] = butterworthFilter = ButterworthFilter(
            name, cutoffFrequency, order, operation, halt, limit, invariant
        )
        return butterworthFilter

    @abaqus_method_doc
    def Chebyshev1Filter(
        self,
        name: str,
        cutoffFrequency: float,
        rippleFactor: float = 0,
        order: int = 2,
        operation: Literal[C.MIN, C.MAX, C.NONE, C.ABS] = NONE,
        halt: Boolean = OFF,
        limit: Optional[float] = None,
        invariant: Literal[C.FIRST, C.SECOND, C.NONE] = NONE,
    ) -> Chebyshev1Filter:
        """This method creates a Chebyshev1Filter object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Chebyshev1Filter
                session.odbs[name].Chebyshev1Filter

        Parameters
        ----------
        name
            A String specifying the repository key. This name ANTIALIASING is reserved for filters
            generated internally by the program.
        cutoffFrequency
            A Float specifying the attenuation point of the filter. Possible values are non-negative
            numbers. Order is not available for OperatorFilter.
        rippleFactor
            A Float specifying the amount of allowable ripple in the filter. Possible values are
            non-negative numbers. The default value is 0.225.
        order
            An Int specifying the highest power of the filter transfer function. Possible values are
            non-negative numbers less than or equal to 20. Order is not available for
            OperatorFilter. The default value is 2.
        operation
            A SymbolicConstant specifying the filter operator. Possible values are NONE, MIN, MAX,
            and ABS. The default value is NONE.
        halt
            A Boolean specifying whether to stop the analysis if the specified limit is reached. The
            default value is OFF.
        limit
            None or a Float specifying the threshold limit, an upper or lower bound for output
            values depending on the operation, or a bound for stopping the analysis when Halt is
            used. The default value is None.
        invariant
            A SymbolicConstant specifying the invariant to which filtering is applied. Possible
            values are NONE, FIRST, and SECOND. The default value is NONE.

        Returns
        -------
        Chebyshev1Filter
            A Chebyshev1Filter object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        self.filters[name] = chebyshev1Filter = Chebyshev1Filter(
            name,
            cutoffFrequency,
            rippleFactor,
            order,
            operation,
            halt,
            limit,
            invariant,
        )
        return chebyshev1Filter

    @abaqus_method_doc
    def Chebyshev2Filter(
        self,
        name: str,
        cutoffFrequency: float,
        rippleFactor: float = 0,
        order: int = 2,
        operation: Literal[C.MIN, C.MAX, C.NONE, C.ABS] = NONE,
        halt: Boolean = OFF,
        limit: Optional[float] = None,
        invariant: Literal[C.FIRST, C.SECOND, C.NONE] = NONE,
    ) -> Chebyshev2Filter:
        """This method creates a Chebyshev2Filter object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Chebyshev2Filter
                session.odbs[name].Chebyshev2Filter

        Parameters
        ----------
        name
            A String specifying the repository key. This name ANTIALIASING is reserved for filters
            generated internally by the program.
        cutoffFrequency
            A Float specifying the attenuation point of the filter. Possible values are non-negative
            numbers. Order is not available for OperatorFilter.
        rippleFactor
            A Float specifying the amount of allowable ripple in the filter. Possible values are
            non-negative numbers less than 1. The default value is 0.025.
        order
            An Int specifying the highest power of the filter transfer function. Possible values are
            non-negative numbers less than or equal to 20. Order is not available for
            OperatorFilter. The default value is 2.
        operation
            A SymbolicConstant specifying the filter operator. Possible values are NONE, MIN, MAX,
            and ABS. The default value is NONE.
        halt
            A Boolean specifying whether to stop the analysis if the specified limit is reached. The
            default value is OFF.
        limit
            None or a Float specifying the threshold limit, an upper or lower bound for output
            values depending on the operation, or a bound for stopping the analysis when Halt is
            used. The default value is None.
        invariant
            A SymbolicConstant specifying the invariant to which filtering is applied. Possible
            values are NONE, FIRST, and SECOND. The default value is NONE.

        Returns
        -------
        Chebyshev2Filter
            A Chebyshev2Filter object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        self.filters[name] = chebyshev2Filter = Chebyshev2Filter(
            name,
            cutoffFrequency,
            rippleFactor,
            order,
            operation,
            halt,
            limit,
            invariant,
        )
        return chebyshev2Filter

    @abaqus_method_doc
    def OperatorFilter(
        self,
        name: str,
        cutoffFrequency: float,
        order: int = 2,
        operation: Literal[C.MIN, C.MAX, C.NONE, C.ABS] = NONE,
        halt: Boolean = OFF,
        limit: Optional[float] = None,
        invariant: Literal[C.FIRST, C.SECOND, C.NONE] = NONE,
    ) -> OperatorFilter:
        """This method creates an OperatorFilter object.

        .. note::
            This function can be accessed by::

                mdb.models[name].OperatorFilter
                session.odbs[name].OperatorFilter

        Parameters
        ----------
        name
            A String specifying the repository key. This name ANTIALIASING is reserved for filters
            generated internally by the program.
        cutoffFrequency
            A Float specifying the attenuation point of the filter. Possible values are non-negative
            numbers. Order is not available for OperatorFilter.
        order
            An Int specifying the highest power of the filter transfer function. Possible values are
            non-negative numbers less than or equal to 20. Order is not available for
            OperatorFilter. The default value is 2.
        operation
            A SymbolicConstant specifying the filter operator. Possible values are NONE, MIN, MAX,
            and ABS. The default value is NONE.
        halt
            A Boolean specifying whether to stop the analysis if the specified limit is reached. The
            default value is OFF.
        limit
            None or a Float specifying the threshold limit, an upper or lower bound for output
            values depending on the operation, or a bound for stopping the analysis when Halt is
            used. The default value is None.
        invariant
            A SymbolicConstant specifying the invariant to which filtering is applied. Possible
            values are NONE, FIRST, and SECOND. The default value is NONE.

        Returns
        -------
        OperatorFilter
            An OperatorFilter object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        self.filters[name] = operatorFilter = OperatorFilter(
            name, cutoffFrequency, order, operation, halt, limit, invariant
        )
        return operatorFilter



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .Interaction import Interaction
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import PRESSURE, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class IncidentWave(Interaction):
    """The IncidentWave object defines incident wave interactions for acoustic and coupled
    acoustic-structural analyses.
    The IncidentWave object is derived from the Interaction object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name]

        The corresponding analysis keywords are:

        - INCIDENT WAVE INTERACTION
    """

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the step in which the IncidentWave object is created.
    createStepName: str

    #: A Region object specifying the incident wave source point.
    sourcePoint: Region

    #: A Region object specifying the incident wave standoff point.This argument is not valid
    #: when **definition** = CONWEP.
    standoffPoint: Region

    #: A Region object specifying the surface defining the incident wave interaction. In
    #: problems involving fluid/surface boundaries, both the fluid surface and the solid
    #: surface comprising the boundary must have an incident wave interaction specified.
    surface: Region

    #: A String specifying the IncidentWaveProperty object associated with this interaction.
    interactionProperty: str

    #: A SymbolicConstant specifying the type of incident wave to be defined. The value must be
    #: PRESSURE for linear perturbation steps. An Explicit step is required when the value is
    #: set to CONWEP. Possible values are PRESSURE, ACCELERATION, UNDEX, and CONWEP. The
    #: default value is PRESSURE.
    definition: SymbolicConstant = PRESSURE

    #: A String specifying the name of the Amplitude object that defines the fluid pressure
    #: time history at the standoff point, if **definition** = PRESSURE. If
    #: **definition** = ACCELERATION, then this string specifies the name of the Amplitude object
    #: that defines the fluid particle acceleration time history at the standoff point. This
    #: member can be specified only if **definition** = PRESSURE or ACCELERATION. The default value
    #: is an empty string.
    amplitude: str = ""

    #: A String specifying the name of the Amplitude object that defines the imaginary
    #: component of the fluid pressure time history at the standoff point. This member is
    #: applicable only for linear perturbation steps and if **definition** = PRESSURE. The default
    #: value is an empty string.
    imaginaryAmplitude: str = ""

    #: A sequence of three Floats specifying the X, Y, and Z components of the direction cosine
    #: of the fluid surface normal.This argument is valid only when **definition** = UNDEX.
    surfaceNormal: tuple = ()

    #: None or a Float specifying the initial depth of the UNDEX bubble. The default value is
    #: None.This argument is valid only when **definition** = UNDEX.
    initialDepth: Optional[float] = None

    #: A Float specifying the reference magnitude.This argument is not valid when
    #: **definition** = CONWEP.
    referenceMagnitude: Optional[float] = None

    #: A Float specifying the time of detonation, given in total time.This argument is valid
    #: only when **definition** = CONWEP.
    detonationTime: Optional[float] = None

    #: A Float specifying the magnitude scale factor. The default value is 1.0.This argument is
    #: valid only when **definition** = CONWEP.
    magnitudeFactor: float = 1

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        sourcePoint: Region,
        standoffPoint: Region,
        surface: Region,
        interactionProperty: str,
        definition: Literal[C.PRESSURE, C.UNDEX, C.CONWEP, C.ACCELERATION] = PRESSURE,
        amplitude: str = "",
        imaginaryAmplitude: str = "",
        surfaceNormal: tuple = (),
        initialDepth: Optional[float] = None,
        referenceMagnitude: Optional[float] = None,
        detonationTime: Optional[float] = None,
        magnitudeFactor: float = 1,
    ):
        """This method creates an IncidentWave object.

        .. note::
            This function can be accessed by::

                mdb.models[name].IncidentWave

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A String specifying the name of the step in which the IncidentWave object is created.
        sourcePoint
            A Region object specifying the incident wave source point.
        standoffPoint
            A Region object specifying the incident wave standoff point.This argument is not valid
            when **definition** = CONWEP.
        surface
            A Region object specifying the surface defining the incident wave interaction. In
            problems involving fluid/surface boundaries, both the fluid surface and the solid
            surface comprising the boundary must have an incident wave interaction specified.
        interactionProperty
            A String specifying the IncidentWaveProperty object associated with this interaction.
        definition
            A SymbolicConstant specifying the type of incident wave to be defined. The value must be
            PRESSURE for linear perturbation steps. An Explicit step is required when the value is
            set to CONWEP. Possible values are PRESSURE, ACCELERATION, UNDEX, and CONWEP. The
            default value is PRESSURE.
        amplitude
            A String specifying the name of the Amplitude object that defines the fluid pressure
            time history at the standoff point, if **definition** = PRESSURE. If
            **definition** = ACCELERATION, then this string specifies the name of the Amplitude object
            that defines the fluid particle acceleration time history at the standoff point. This
            member can be specified only if **definition** = PRESSURE or ACCELERATION. The default value
            is an empty string.
        imaginaryAmplitude
            A String specifying the name of the Amplitude object that defines the imaginary
            component of the fluid pressure time history at the standoff point. This member is
            applicable only for linear perturbation steps and if **definition** = PRESSURE. The default
            value is an empty string.
        surfaceNormal
            A sequence of three Floats specifying the X, Y, and Z components of the direction cosine
            of the fluid surface normal.This argument is valid only when **definition** = UNDEX.
        initialDepth
            None or a Float specifying the initial depth of the UNDEX bubble. The default value is
            None.This argument is valid only when **definition** = UNDEX.
        referenceMagnitude
            A Float specifying the reference magnitude.This argument is not valid when
            **definition** = CONWEP.
        detonationTime
            A Float specifying the time of detonation, given in total time.This argument is valid
            only when **definition** = CONWEP.
        magnitudeFactor
            A Float specifying the magnitude scale factor. The default value is 1.0.This argument is
            valid only when **definition** = CONWEP.

        Returns
        -------
        IncidentWave
            An IncidentWave object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        definition: Literal[C.PRESSURE, C.UNDEX, C.CONWEP, C.ACCELERATION] = PRESSURE,
        amplitude: str = "",
        imaginaryAmplitude: str = "",
        surfaceNormal: tuple = (),
        initialDepth: Optional[float] = None,
        referenceMagnitude: Optional[float] = None,
        detonationTime: Optional[float] = None,
        magnitudeFactor: float = 1,
    ):
        """This method modifies the IncidentWave object.

        Parameters
        ----------
        definition
            A SymbolicConstant specifying the type of incident wave to be defined. The value must be
            PRESSURE for linear perturbation steps. An Explicit step is required when the value is
            set to CONWEP. Possible values are PRESSURE, ACCELERATION, UNDEX, and CONWEP. The
            default value is PRESSURE.
        amplitude
            A String specifying the name of the Amplitude object that defines the fluid pressure
            time history at the standoff point, if **definition** = PRESSURE. If
            **definition** = ACCELERATION, then this string specifies the name of the Amplitude object
            that defines the fluid particle acceleration time history at the standoff point. This
            member can be specified only if **definition** = PRESSURE or ACCELERATION. The default value
            is an empty string.
        imaginaryAmplitude
            A String specifying the name of the Amplitude object that defines the imaginary
            component of the fluid pressure time history at the standoff point. This member is
            applicable only for linear perturbation steps and if **definition** = PRESSURE. The default
            value is an empty string.
        surfaceNormal
            A sequence of three Floats specifying the X, Y, and Z components of the direction cosine
            of the fluid surface normal.This argument is valid only when **definition** = UNDEX.
        initialDepth
            None or a Float specifying the initial depth of the UNDEX bubble. The default value is
            None.This argument is valid only when **definition** = UNDEX.
        referenceMagnitude
            A Float specifying the reference magnitude.This argument is not valid when
            **definition** = CONWEP.
        detonationTime
            A Float specifying the time of detonation, given in total time.This argument is valid
            only when **definition** = CONWEP.
        magnitudeFactor
            A Float specifying the magnitude scale factor. The default value is 1.0.This argument is
            valid only when **definition** = CONWEP.
        """
        ...



from typing import Union, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .Constraint import Constraint
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import Boolean, OFF, ON, SymbolicConstant, UNIFORM
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class Coupling(Constraint):
    """The Coupling object defines a constraint between a group of coupling nodes located on a
    region and a reference point.
    The Coupling object is derived from the ConstrainedSketchConstraint object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].constraints[name]

        The corresponding analysis keywords are:

        - COUPLING
    """

    #: A Boolean specifying whether the constraint is suppressed or not. The default value is
    #: OFF.
    suppressed: Boolean = OFF

    #: A String specifying the constraint repository key.
    name: str

    #: A Region object specifying the surface on which the coupling nodes are located.
    surface: Region

    #: A Region object specifying the constraint control point.
    controlPoint: Region

    #: The SymbolicConstant WHOLE_SURFACE or a Float specifying the influence radius.
    influenceRadius: Union[SymbolicConstant, float]

    #: A SymbolicConstant specifying the coupling constraint type. Possible values are
    #: KINEMATIC, DISTRIBUTING, and STRUCTURAL.
    couplingType: SymbolicConstant

    #: A Boolean specifying if the control point will be adjusted (moved) to the surface. The
    #: point will be adjusted in the direction normal to the specified surface. The default
    #: value is OFF.
    adjust: Boolean = OFF

    #: None or a DatumCsys object specifying the initial orientation of the local coordinate
    #: system for the coupling's degrees of freedom. If **localCsys** = None, the coupling is
    #: defined in the global coordinate system. The default value is None.
    localCsys: Optional[str] = None

    #: A Boolean specifying if the displacement component in the 1-direction is constrained to
    #: the reference node for a kinematic coupling constraint. The default value is ON.The **u1**
    #: argument applies only when **couplingType** = KINEMATIC.
    u1: Boolean = ON

    #: A Boolean specifying if the displacement component in the 2-direction is constrained to
    #: the reference node for a kinematic coupling constraint. The default value is ON.The **u2**
    #: argument applies only when **couplingType** = KINEMATIC.
    u2: Boolean = ON

    #: A Boolean specifying if the displacement component in the 3-direction is constrained to
    #: the reference node for a kinematic coupling constraint. The default value is ON.The **u3**
    #: argument applies only when **couplingType** = KINEMATIC.
    u3: Boolean = ON

    #: A Boolean specifying if the rotational displacement component about the 1-direction is
    #: constrained to the reference node for a kinematic coupling constraint. The default value
    #: is ON.The **ur1** argument applies only when **couplingType** = KINEMATIC.
    ur1: Boolean = ON

    #: A Boolean specifying if the rotational displacement component about the 2-direction is
    #: constrained to the reference node for a kinematic coupling constraint. The default value
    #: is ON.The **ur2** argument applies only when **couplingType** = KINEMATIC.
    ur2: Boolean = ON

    #: A Boolean specifying if the rotational displacement component about the 3-direction is
    #: constrained to the reference node for a kinematic coupling constraint. The default value
    #: is ON.The **ur3** argument applies only when **couplingType** = KINEMATIC.
    ur3: Boolean = ON

    #: A SymbolicConstant specifying an optional weighting method used for calculating the
    #: distributing weight factors. Possible values are UNIFORM, LINEAR, QUADRATIC, and CUBIC.
    #: The default value is UNIFORM.The **weightingMethod** argument applies only when
    #: **couplingType** = DISTRIBUTING.
    weightingMethod: SymbolicConstant = UNIFORM

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        surface: Region,
        controlPoint: Region,
        influenceRadius: Union[Literal[C.WHOLE_SURFACE], float],
        couplingType: Literal[C.STRUCTURAL, C.DISTRIBUTING, C.KINEMATIC],
        adjust: Boolean = OFF,
        localCsys: Optional[str] = None,
        u1: Boolean = ON,
        u2: Boolean = ON,
        u3: Boolean = ON,
        ur1: Boolean = ON,
        ur2: Boolean = ON,
        ur3: Boolean = ON,
        weightingMethod: Literal[C.QUADRATIC, C.DISTRIBUTING, C.UNIFORM, C.LINEAR, C.CUBIC] = UNIFORM,
    ):
        """This method creates a Coupling object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Coupling

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        surface
            A Region object specifying the surface on which the coupling nodes are located.
        controlPoint
            A Region object specifying the constraint control point.
        influenceRadius
            The SymbolicConstant WHOLE_SURFACE or a Float specifying the influence radius.
        couplingType
            A SymbolicConstant specifying the coupling constraint type. Possible values are
            KINEMATIC, DISTRIBUTING, and STRUCTURAL.
        adjust
            A Boolean specifying if the control point will be adjusted (moved) to the surface. The
            point will be adjusted in the direction normal to the specified surface. The default
            value is OFF.
        localCsys
            None or a DatumCsys object specifying the initial orientation of the local coordinate
            system for the coupling's degrees of freedom. If **localCsys** = None, the coupling is
            defined in the global coordinate system. The default value is None.
        u1
            A Boolean specifying if the displacement component in the 1-direction is constrained to
            the reference node for a kinematic coupling constraint. The default value is ON.The **u1**
            argument applies only when **couplingType** = KINEMATIC.
        u2
            A Boolean specifying if the displacement component in the 2-direction is constrained to
            the reference node for a kinematic coupling constraint. The default value is ON.The **u2**
            argument applies only when **couplingType** = KINEMATIC.
        u3
            A Boolean specifying if the displacement component in the 3-direction is constrained to
            the reference node for a kinematic coupling constraint. The default value is ON.The **u3**
            argument applies only when **couplingType** = KINEMATIC.
        ur1
            A Boolean specifying if the rotational displacement component about the 1-direction is
            constrained to the reference node for a kinematic coupling constraint. The default value
            is ON.The **ur1** argument applies only when **couplingType** = KINEMATIC.
        ur2
            A Boolean specifying if the rotational displacement component about the 2-direction is
            constrained to the reference node for a kinematic coupling constraint. The default value
            is ON.The **ur2** argument applies only when **couplingType** = KINEMATIC.
        ur3
            A Boolean specifying if the rotational displacement component about the 3-direction is
            constrained to the reference node for a kinematic coupling constraint. The default value
            is ON.The **ur3** argument applies only when **couplingType** = KINEMATIC.
        weightingMethod
            A SymbolicConstant specifying an optional weighting method used for calculating the
            distributing weight factors. Possible values are UNIFORM, LINEAR, QUADRATIC, and CUBIC.
            The default value is UNIFORM.The **weightingMethod** argument applies only when
            **couplingType** = DISTRIBUTING.

        Returns
        -------
        Coupling
            A Coupling object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        adjust: Boolean = OFF,
        localCsys: Optional[str] = None,
        u1: Boolean = ON,
        u2: Boolean = ON,
        u3: Boolean = ON,
        ur1: Boolean = ON,
        ur2: Boolean = ON,
        ur3: Boolean = ON,
        weightingMethod: Literal[C.QUADRATIC, C.DISTRIBUTING, C.UNIFORM, C.LINEAR, C.CUBIC] = UNIFORM,
    ):
        """This method modifies the Coupling object.

        Parameters
        ----------
        adjust
            A Boolean specifying if the control point will be adjusted (moved) to the surface. The
            point will be adjusted in the direction normal to the specified surface. The default
            value is OFF.
        localCsys
            None or a DatumCsys object specifying the initial orientation of the local coordinate
            system for the coupling's degrees of freedom. If **localCsys** = None, the coupling is
            defined in the global coordinate system. The default value is None.
        u1
            A Boolean specifying if the displacement component in the 1-direction is constrained to
            the reference node for a kinematic coupling constraint. The default value is ON.The **u1**
            argument applies only when **couplingType** = KINEMATIC.
        u2
            A Boolean specifying if the displacement component in the 2-direction is constrained to
            the reference node for a kinematic coupling constraint. The default value is ON.The **u2**
            argument applies only when **couplingType** = KINEMATIC.
        u3
            A Boolean specifying if the displacement component in the 3-direction is constrained to
            the reference node for a kinematic coupling constraint. The default value is ON.The **u3**
            argument applies only when **couplingType** = KINEMATIC.
        ur1
            A Boolean specifying if the rotational displacement component about the 1-direction is
            constrained to the reference node for a kinematic coupling constraint. The default value
            is ON.The **ur1** argument applies only when **couplingType** = KINEMATIC.
        ur2
            A Boolean specifying if the rotational displacement component about the 2-direction is
            constrained to the reference node for a kinematic coupling constraint. The default value
            is ON.The **ur2** argument applies only when **couplingType** = KINEMATIC.
        ur3
            A Boolean specifying if the rotational displacement component about the 3-direction is
            constrained to the reference node for a kinematic coupling constraint. The default value
            is ON.The **ur3** argument applies only when **couplingType** = KINEMATIC.
        weightingMethod
            A SymbolicConstant specifying an optional weighting method used for calculating the
            distributing weight factors. Possible values are UNIFORM, LINEAR, QUADRATIC, and CUBIC.
            The default value is UNIFORM.The **weightingMethod** argument applies only when
            **couplingType** = DISTRIBUTING.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..TableCollection.ParameterColumn import ParameterColumn
from ..TableCollection.ParameterColumnArray import ParameterColumnArray
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class ParameterTable:
    """A ParameterTable is an object that is used to define the containers that encapsulate
    ParameterColumn and DataTable objects. The data of DataTable is dependent on the
    contents of ParameterColumn. After DataTable is instantiated, making changes to
    ParameterColumn may lead to data corruption.

    .. note::
        This object can be accessed by::

            mdb.models[name].tableCollections[name].parameterTables[name]

        The corresponding analysis keywords are:

        - PARAMETER TABLE TYPE
        - PARAMETER TABLE

    .. versionadded:: 2020
        The `ParameterTable` class was added.
    """

    #: A ParameterColumnArray specifying all the columns in the ParameterTable.
    columns: ParameterColumnArray = []

    #: A DataTableArray specifying all the dataTables in the ParameterTable.
    dataTables: str = ""

    @abaqus_method_doc
    def __init__(self, name: str):
        """This method creates a ParameterTable object and places it in the parameterTables
        repository.

        .. note::
            This function can be accessed by::

                mdb.models[name].tableCollections[name].ParameterTable

        Parameters
        ----------
        name
            A String specifying the repository key.

        Returns
        -------
        ParameterTable
            A ParameterTable object.
        """
        ...

    @abaqus_method_doc
    def Column(
        self,
        type: Literal[C.INTEGER, C.FLOAT, C.STRING],
        unit: str = "",
        description: str = "",
        default: str = "",
        allowedValues: str = "",
    ) -> ParameterColumn:
        """ParameterColumn is a constructor method that creates a ParameterColumn object and stores
        it in the array data structure. It is accessible from the ParameterTable object using a
        column member.

        .. note::
            This function can be accessed by::

                mdb.models[name].tableCollections[name].ParameterTable

        Parameters
        ----------
        type
            A SymbolicConstant specifying the data type of the parameter. Possible values are
            STRING, INTEGER, and FLOAT.
        unit
            A String specifying the unit of the parameter.
        description
            A String specifying the description of the parameter.
        default
            The default value of the first parameter. The data type of the value depends on the
            value of **type** argument.
        allowedValues
            A Set of allowed values for the parameter.

        Returns
        -------
        ParameterTable
            A ParameterTable object.

        Raises
        ------
        Incompatible data are given
        """
        parameterColumn = ParameterColumn(type, unit, description, default, allowedValues)
        self.columns.append(parameterColumn)
        return parameterColumn



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .ConcreteCompressionDamage import ConcreteCompressionDamage
from .ConcreteCompressionHardening import ConcreteCompressionHardening
from .ConcreteTensionDamage import ConcreteTensionDamage
from .ConcreteTensionStiffening import ConcreteTensionStiffening
from ....UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class ConcreteDamagedPlasticity:
    r"""The ConcreteDamagedPlasticity object specifies the concrete damaged plasticity model.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].concreteDamagedPlasticity
            import odbMaterial
            session.odbs[name].materials[name].concreteDamagedPlasticity

        The table data for this object are:

        - Dilation angle, :math:`\psi` (in degrees) in the :math:`p - q` plane.
        - Flow potential eccentricity, :math:`\epsilon`. The default value is 0.1.
        - :math:`\sigma_{b 0} / \sigma_{t 0}`, the ratio of initial equibiaxial compressive
          yield stress to initial uniaxial compressive yield stress. The default value
          is 1.16. stress is negative. The default value is 2/3. default value is 0.0.
        - :math:`K_c`, the ratio of the second stress invariant on the tensile meridian, to that on
          the compressive meridian, at initial yield for any given value of the pressure invariant :math:`p`
          such that the maximum principal stress is negative. The default value is 2/3.
        - Viscosity parameter, :math:`\mu`, used for the viscoplastic regularization of the concrete
          constitutive equations in an Abaqus/Standard analysis. This parameter is ignored in an
          Abaqus/Explicit analysis. The default value is 0.0.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - CONCRETE DAMAGED PLASTICITY
    """

    #: A ConcreteCompressionHardening object.
    concreteCompressionHardening: ConcreteCompressionHardening = ConcreteCompressionHardening(((),))

    #: A ConcreteTensionStiffening object.
    concreteTensionStiffening: ConcreteTensionStiffening = ConcreteTensionStiffening(((),))

    #: A ConcreteCompressionDamage object.
    concreteCompressionDamage: ConcreteCompressionDamage = ConcreteCompressionDamage(((),))

    #: A ConcreteTensionDamage object.
    concreteTensionDamage: ConcreteTensionDamage = ConcreteTensionDamage(((),))

    @abaqus_method_doc
    def __init__(self, table: tuple, temperatureDependency: Boolean = OFF, dependencies: int = 0):
        """This method creates a ConcreteDamagedPlasticity object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].ConcreteDamagedPlasticity
                session.odbs[name].materials[name].ConcreteDamagedPlasticity

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        ConcreteDamagedPlasticity
            A ConcreteDamagedPlasticity object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ConcreteDamagedPlasticity object.

        Raises
        ------
        RangeError
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .NumberFormat import NumberFormat
from ..UtilityAndView.abaqusConstants import Boolean, COMPRESSEDINDEX, ENGINEERING, ON
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class JournalOptions:
    """A JournalOptions object specifies how to record selection of geometry in the journal and
    replay files. **journalOptions** can also be used to set the numeric formatting options
    for field report output, geometry commands output, and a default format for other
    numeric output. The JournalOptions object has no constructor. Abaqus creates the
    **journalOptions** member when a session is started.

    .. note::
        This object can be accessed by::

            session.journalOptions
    """

    #: Format of the number
    numberFormat: NumberFormat = NumberFormat()

    @abaqus_method_doc
    def setValues(
        self,
        replayGeometry: Literal[C.COORDINATE, C.COMPRESSEDINDEX, C.INDEX] = COMPRESSEDINDEX,
        recoverGeometry: Literal[C.COORDINATE, C.COMPRESSEDINDEX, C.INDEX] = COMPRESSEDINDEX,
        defaultFormat: Optional[NumberFormat] = None,
        fieldReportFormat: Optional[NumberFormat] = None,
        geometryFormat: Optional[NumberFormat] = None,
    ):
        """This method modifies the JournalOptions object.

        Parameters
        ----------
        replayGeometry
            A SymbolicConstant specifying the format of the geometry in the replay file. Possible
            values are COORDINATE, INDEX, and COMPRESSEDINDEX. The default value is COMPRESSEDINDEX.
        recoverGeometry
            A SymbolicConstant specifying the format of the geometry in the recovery file. Possible
            values are COORDINATE, INDEX, and COMPRESSEDINDEX. The default value is COMPRESSEDINDEX.
        defaultFormat
            A NumberFormat object specifying the default format for numeric output. The default
            values are the same as the default values for the NumberFormat object.
        fieldReportFormat
            A NumberFormat object specifying the default format for numbers in a field report
            output. The default values are the same as the default values for the NumberFormat
            object.
        geometryFormat
            A NumberFormat object specifying the default format for numbers in geometry commands
            output. The default values are the same as the default values for the NumberFormat
            object.
        """
        ...

    @abaqus_method_doc
    def NumberFormat(
        self,
        blankPad: Boolean = ON,
        format: Literal[C.AUTOMATIC, C.ENGINEERING, C.SCIENTIFIC] = ENGINEERING,
        numDigits: int = 6,
        precision: int = 0,
    ) -> NumberFormat:
        """This method creates a NumberFormat object.

        .. note::
            This function can be accessed by::

                session.defaultFieldReportOptions.NumberFormat
                session.fieldReportOptions.NumberFormat
                session.journalOptions.NumberFormat

        Parameters
        ----------
        blankPad
            A Boolean specifying whether the printed digits should be padded with blank characters
            to ensure equal sized fields. The **blankPad** argument is useful when your printed output
            includes columns. The default value is ON.
        format
            A SymbolicConstant specifying the formatting type. Possible values are ENGINEERING,
            SCIENTIFIC, and AUTOMATIC. The default value is ENGINEERING.
        numDigits
            An Int specifying the number of digits to be displayed in the result. **numDigits** >0>0.
            The default value is 6.
        precision
            An Int specifying the number of decimal places to which the number is to be truncated
            for display. **precision** ≤0≤0. If **precision** =0, no truncation is applied. The default
            value is 0.

        Returns
        -------
        NumberFormat
            A NumberFormat object.
        """
        self.numberFormat = numberFormat = NumberFormat(blankPad, format, numDigits, precision)
        return numberFormat



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import Boolean, NONE, OFF, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class ElementProgressiveActivation:
    """The ElementProgressiveActivation object is used to specify elements that can be
    activated during an analysis.

    .. note::
        This object can be accessed by::

            mdb.models[name].rootAssembly.elementProgressiveActivations[name]

        The corresponding analysis keywords are:

        - ELEMENT PROGRESSIVE ACTIVATION

    .. versionadded:: 2020
        The `ElementProgressiveActivation` class was added.
    """

    #: A String specifying the key of the repository.
    name: str

    #: A Region object specifying the region containing the elements that will be activated
    #: during the analysis.
    elset: Region

    #: A Boolean value specifying whether the elements that have not yet been activated will
    #: follow the deformations of the active elements. Set **deformation** = ON when the
    #: deformation of the active elements is excessive. The default value is OFF.
    deformation: Boolean = OFF

    #: A SymbolicConstant specifying the exposed areas of the element facets that are active
    #: for convection or radiation boundary conditions to be applied. Possible values are NONE
    #: and FACET. If **freeSurfaceType** = FACET, user subroutine UEPACTIVATIONFACET will be called
    #: at the start of the increment for each element. If **freeSurfaceType** = NONE, all the
    #: exposed areas of the element facets are considered. The default value is NONE.
    freeSurfaceType: SymbolicConstant = NONE

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        elset: Region,
        deformation: Boolean = OFF,
        freeSurfaceType: Literal[C.FACET, C.NONE] = NONE,
    ):
        """This method creates an ElementProgressiveActivation object and places it in the
        elementProgressiveActivation repository.

        .. note::
            This function can be accessed by::

                mdb.models[name].rootAssembly.ElementProgressiveActivation

        Parameters
        ----------
        name
            A String specifying the key of the repository.
        elset
            A Region object specifying the region containing the elements that will be activated
            during the analysis.
        deformation
            A Boolean value specifying whether the elements that have not yet been activated will
            follow the deformations of the active elements. Set **deformation** = ON when the
            deformation of the active elements is excessive. The default value is OFF.
        freeSurfaceType
            A SymbolicConstant specifying the exposed areas of the element facets that are active
            for convection or radiation boundary conditions to be applied. Possible values are NONE
            and FACET. If **freeSurfaceType** = FACET, user subroutine UEPACTIVATIONFACET will be called
            at the start of the increment for each element. If **freeSurfaceType** = NONE, all the
            exposed areas of the element facets are considered. The default value is NONE.

        Returns
        -------
        ElementProgressiveActivation
            An ElementProgressiveActivation object.

        Raises
        ------
        AbaqusException
            If the region does not contain only elements.
        """
        ...

    @abaqus_method_doc
    def setValue(self):
        """The method modifies the ElementProgressiveActivation object.

        Returns
        -------

        Raises
        ------
        """
        ...



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

import inspect
import os
import re
import sys
import typing

import git

project = 'abqpy'
copyright = '2022, WANG Hailin'
author = 'WANG Hailin'

# The full version, including alpha/beta/rc tags
try:
    import abqpy
    release = version = abqpy.__semver__[:4]
except (ImportError, AttributeError):
    import warnings
    warnings.warn('abqpy is not installed, using 2020.0.0')
    release = version = '2020.0.0'[:4]
sys.path.insert(0, os.path.abspath('../../src'))
sys.path.insert(0, os.path.abspath('./_ext'))

# For multiple languages
locale_dirs = ['locales/']   # path is example but recommended.
gettext_compact = False      # optional.

# -- General configuration ---------------------------------------------------

# Environment variables
os.environ['ABQPY_MAKE_DOCS'] = 'true'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx_toolbox.more_autodoc.overloads',
    'autoclasstoc',
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.linkcode',
    'sphinx.ext.githubpages',
    'sphinx.ext.intersphinx',
    'sphinx.ext.mathjax',
    'sphinx.ext.napoleon',
    'sphinx_copybutton',
    'sphinx_autodoc_toolbox.collapse',
    'sphinx_autodoc_typehints',
    'sphinx_codeautolink',
    'sphinx_design',
    'sphinx_gallery.gen_gallery',
    'sphinx_togglebutton',
    'sphinx_toolbox.confval',
    'sphinx_toolbox.collapse',
    'hoverxref.extension',
    'myst_parser',
    'autopkg',
    'automembers',
]

# automembers configuration
automembers_autodoc_options = [
    ":members:",
    ":special-members: __init__",
    ":show-inheritance:",
]

# MyST configuration
myst_enable_extensions = [
  "colon_fence",
]

# Default behavior for code block concatenation for sphinx_codeautolink
codeautolink_concat_default = False

# Suppress warnings
suppress_warnings = [
    'app.add_directive',
]

intersphinx_mapping = {
    'jinjia2': ('https://jinja.palletsprojects.com/en/3.0.x/', None),
    'matplotlib': ('https://matplotlib.org/stable/', None),
    'numpy': ('https://numpy.org/doc/stable/', None),
    'pandas': ('https://pandas.pydata.org/pandas-docs/stable/', None),
    'pytest': ('https://pytest.org/en/stable/', None),
    'python': ('https://docs.python.org/3/', None),
    'readthedocs': ('https://docs.readthedocs.io/en/stable/', None),
    'scipy': ('https://docs.scipy.org/doc/scipy/', None),
}

# Hoverxref configuration
hoverxref_auto_ref = True
hoverxref_domains = ["py"]
hoverxref_roles = [
    'numref',
    'confval',
    'setting',
    "option",
    "doc",  # Documentation pages
    "term",  # Glossary terms
]
hoverxref_role_types = {
    "doc": "modal",  # for whole docs
    "mod": "modal",  # for Python Sphinx Domain
    "class": "tooltip",  # for Python Sphinx Domain
    "func": "tooltip",  # for Python Sphinx Domain
    "meth": "tooltip",  # for Python Sphinx Domain
    "attr": "tooltip",  # for Python Sphinx Domain
    "exc": "tooltip",  # for Python Sphinx Domain
    "obj": "tooltip",  # for Python Sphinx Domain
    "ref": "tooltip",  # for hoverxref_auto_ref config
    "confval": "tooltip",  # for custom object
    "term": "tooltip",  # for glossaries
    "numref": "tooltip", 
}
hoverxref_intersphinx = [
    'numpy',
    'pytest',
    'python',
    'readthedocs',
]

# Sphinx gallery configuration
sphinx_gallery_conf = {
     'examples_dirs': '../../examples',
     'gallery_dirs': 'examples',
     'filename_pattern': '/.+\.py',
     'plot_gallery': False,
}

# Show short type hints for user-defined classes and defaults for parameters
python_use_unqualified_type_names = True
autodoc_typehints_format = 'short'
typehints_defaults = 'comma'
typehints_document_rtype = False
autodoc_default_options = {
    'undoc-members': False,
}

add_module_names = False

# Figure numbering
numfig = True

# True to convert the type definitions in the docstrings as references. Defaults to False.
napoleon_preprocess_types = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'pydata_sphinx_theme'

# Logo
# html_logo = "_static/3ds-dark.svg"

html_sidebars = {
    "**": ["search-field.html", "sidebar-nav-bs.html"]
}

html_theme_options = {
    'navigation_depth': 9,
    "github_url": "https://github.com/haiiliin/abqpy",
    "icon_links": [
        {
            "name": "PyPI",
            "url": "https://pypi.org/project/abqpy",
            "icon": "_static/PyPI_logo.svg",
            "type": "local",
        },
        {
            "name": "Read the Docs",
            "url": "https://readthedocs.org/projects/abqpy/",
            "icon": "_static/rtd-logo-dark.svg",
            "type": "local",
        },
   ],
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# -- Options for LaTeX output -------------------------------------------------

# If true, add page references after internal references. This is very useful
# for printed copies of the manual. Default is False.
latex_show_pagerefs = True

# Control whether to display URL addresses. This is very useful for printed
# copies of the manual. The setting can have the following values:
# 'no' - do not display URLs (default)
# 'footnote' - display URLs in footnotes
# 'inline' - display URLs inline in parentheses
latex_show_urls = 'footnote'

# If given, this must be the name of an image file (relative to the
# configuration directory) that is the logo of the docs. It is placed at the
# top of the title page. Default: None.
latex_logo = None

# The “theme” that the LaTeX output should use. It is a collection of settings
# for LaTeX output (ex. document class, top level sectioning unit and so on).
# As a built-in LaTeX themes, manual and howto are bundled.
# manual
# A LaTeX theme for writing a manual. It imports the report document class
# (Japanese documents use jsbook).
# howto
# A LaTeX theme for writing an article. It imports the article document class
# (Japanese documents use jreport rather). latex_appendices is available only for this theme.
# It defaults to 'manual'.
latex_theme = 'manual'

# autoclass_content = 'both'

latex_toplevel_sectioning = 'part'
latex_engine = 'xelatex'
latex_use_xindy = False
latex_elements = {
    'preamble': '\\usepackage[UTF8]{ctex}\n\\setcounter{tocdepth}{3}\n\\setcounter{secnumdepth}{5}',
    'printindex': '\\def\\twocolumn[#1]{#1}\\printindex',
}


# linkcode source
def linkcode_resolve(domain: str, info: dict[str, typing.Union[str, list[str]]]):
    """Resolve linkcode source
    
    Parameters
    ----------
    domain : str
        specifies the language domain the object is in
    info : dict[str, str | list[str]]
        a dictionary with the following keys guaranteed to be present (dependent on the domain)

        - py: module (name of the module), fullname (name of the object)
        - c: names (list of names for the object)
        - cpp: names (list of names for the object)
        - javascript: object (name of the object), fullname (name of the item)

    Returns
    -------
    source url of the object
    """
    if domain != 'py':
        return None

    modname = info['module']
    fullname = info['fullname']

    filename = modname.replace('.', '/')
    try:
        branch_name = git.repo.Repo('../../').active_branch.name
    except Exception:
        branch_name = version[:4]
    baseurl = f'https://github.com/haiiliin/abqpy/blob/{branch_name}/src/{filename}.py'

    submod = sys.modules.get(modname)
    if submod is None:
        return baseurl

    obj = submod
    for part in fullname.split('.'):
        try:
            obj = getattr(obj, part)
        except Exception:
            return baseurl
    try:
        source, lineno = inspect.getsourcelines(obj)
    except TypeError:
        # Find source line for an attribute, the obj is None
        attr = fullname.split('.')[-1]
        obj = submod
        for part in fullname.split('.')[:-1]:
            try:
                obj = getattr(obj, part)
            except Exception:
                return baseurl
        source, lineno = inspect.getsourcelines(obj)
        attr_sources: list[str] = re.findall(rf'\n(    {attr}: [\w\W]+?)\n\n', '\n'.join(source))
        if len(attr_sources) > 0:
            attr_source = attr_sources[0].splitlines()

            def find_line_number(string: str, text: list[str]):
                for line_number, line in enumerate(text):
                    if string in line:
                        return line_number

            index = find_line_number(attr_source[0], source)
            for row in range(index - 1, -1, -1):
                if source[row].startswith('    #: '):
                    attr_source.insert(0, source[row])
                else:
                    break
            lineno += find_line_number(attr_source[0], source)
            source = attr_source
    except Exception:
        return baseurl

    return baseurl + f'#L{lineno}-L{lineno + len(source) - 1}'



import os
import re
import sys

from abaqus import Mdb, backwardCompatibility, getInput, mdb
from abaqus.Mdb.MdbCommands import upgradeMdb, CombineOptResults, openMdb  # noqa # pylint: disable=unused-import
from abaqus.Session.Session import Session
from abaqus.UtilityAndView import SymbolicConstant, BackwardCompatibility
from abaqus.UtilityAndView.AbaqusBoolean import AbaqusBoolean
from abaqus.UtilityAndView.Repository import Repository
from abaqus.UtilityAndView.abaqusConstants import (AbaqusBooleanType, Boolean, BooleanType, CFD, EMAG, FALSE, NOT_SET,
                                                   OFF, ON, SCATTERED, STANDARD_EXPLICIT, STEP_END,
                                                   SymbolicConstantType, TOTAL, TRUE)

NoneType = type(None)


MdbType = type(Mdb)
BackwardCompatibilityType = type(BackwardCompatibility)
SessionType = type(Session)
RepositoryType = type(Repository)


# Inspected from Abaqus/CAE 2021 Command Line Interface
__all__ = [
    'AbaqusBoolean',
    'AbaqusBooleanType',
    # 'AbaqusException',
    # 'AbaqusShutdown',
    'BackwardCompatibilityType',
    'Boolean',
    'BooleanType',
    'CFD',
    'EMAG',
    'FALSE',
    'Mdb',
    'MdbType',
    'NOT_SET',
    'NoneType',
    'OFF',
    'ON',
    'RepositoryType',
    'SCATTERED',
    'STANDARD_EXPLICIT',
    'STEP_END',
    'SessionType',
    'SymbolicConstant',
    'SymbolicConstantType',
    'TOTAL',
    'TRUE',
    # '__builtins__',
    # '__doc__',
    # '__name__',
    # '__package__',
    # '_fireQuery',
    # '_getCurrentExecFileCmd',
    # '_getDebug',
    # '_getMdbSchema',
    # '_getMdbUpToDateStatus',
    # '_getShowDisplaysState',
    # '_getShowLandmarksState',
    # '_getShowVerbosesState',
    # '_getShowWarningsState',
    # '_getWarningReply',
    # '_journalMethodCallInternal',
    # '_listDebug',
    # '_pprint',
    # '_printToCli',
    # '_replayCapture',
    # '_setCallVerifyMdb',
    # '_setDebug',
    # '_setImportModuleHook',
    # '_setMessagesState',
    # '_shutdowm',
    # '_upgradeTableMdb',
    # 'addCmdToCommandHistory',
    # 'applicationName',
    'backwardCompatibility',
    # 'compress',
    # 'copyrightYear',
    'getInput',
    # 'getInputs',
    # 'getMdbVersionNumbers',
    # 'getReplayFileName',
    # 'isMdbChanged',
    # 'isMdbStudentEdition',
    # 'isStudentEdition',
    # 'journalMethodDecrement',
    # 'journalMethodIncrement',
    # 'landmark',
    # 'majorVersion',
    'mdb',
    # 'mdbVersion',
    # 'milestone',
    # 'minorVersion',
    # 'odbVersion',
    # 'openMdb',
    'os',
    're',
    # 'showStopButtonInGui',
    'sys',
    # 'uncompress',
    # 'updateVersion',
    # 'upgradeMdb',
    # 'version'
]



import ast
import os
import sys
from typing import Dict, Union

ABAQUS_COMMAND_OPTIONS = {"noGUI": True}


def run(cae: bool = True) -> None:
    """Runs Abaqus command in system's CLI

    This function uses the top level script file to run the Abaqus
    command, and the arguments passed to it

    Parameters
    ----------
    cae : bool, optional
        Whether to use `abaqus cae` command, True for `abaqus cae`, False for `abaqus python`, by default True
    """
    abaqus = os.environ.get("ABAQUS_BAT_PATH", "abaqus")
    main = sys.modules["__main__"]
    if not hasattr(main, "__file__") or main.__file__ is None:
        return

    filePath = os.path.relpath(main.__file__)
    args = " ".join(sys.argv[1:])

    try:  # If it is a jupyter notebook
        import ipynbname

        filePath = ipynbname.path()
        os.system(f"jupyter nbconvert --to python {filePath}")
        filePath = str(filePath).replace(".ipynb", ".py")
    except (FileNotFoundError, ImportError, Exception):
        pass

    # check if in debug mode and run
    debug = os.environ.get("ABQPY_DEBUG", "false").lower() == "true"
    gettrace = getattr(sys, "gettrace", None)
    debug = debug or (gettrace is not None and gettrace())

    # Check if it is imported by sphinx to generate docs
    make_docs = os.environ.get("ABQPY_MAKE_DOCS", "false").lower() == "true"

    # Alternative to use abaqus command line options at run time
    abq_cmd_opt: Dict[str, Union[str, bool]] = ast.literal_eval(
        os.environ.get("ABAQUS_COMMAND_OPTIONS", str(ABAQUS_COMMAND_OPTIONS))
    )

    if cae:
        proc = "cae"
        mode = f'noGUI="{filePath}"' if abq_cmd_opt.pop("noGUI", True) else f'script="{filePath}"'
        sep = "--" if args else ""
    else:
        proc = "python"
        mode = f'"{filePath}"'
        sep = ""

    options = [
        f"{key}={value}" if isinstance(value, str) else f"{key}" if value else "" for key, value in abq_cmd_opt.items()
    ]

    # If in debug mode do not run the abaqus command at all
    if not debug and not make_docs:
        cmd = f"{abaqus} {proc} {mode} {' '.join(options)} {sep} {' '.join(args)}".strip()
        message = f"Running the following abaqus command: {cmd}"
        print("", "-" * len(message), message, "-" * len(message), sep="\n")
        os.system(cmd)
        sys.exit(0)



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region


@abaqus_class_doc
class AdaptiveMeshDomain:
    """The AdaptiveMeshDomain object defines the region and controls that govern an Arbitrary
    Lagrangian Eularian (ALE) style adaptive smoothing mesh domain.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].adaptiveMeshDomains[name]
    """

    #: A Region object specifying the region to which the adaptive mesh domain is applied.
    region: Region

    #: A String specifying the name of an AdaptiveMeshControl object.
    controls: str = ""

    #: An Int specifying the frequency in increments at which adaptive meshing will be
    #: performed. The default value is 10.
    frequency: int = 10

    #: An Int specifying the number of mesh sweeps to be performed at the beginning of the
    #: first step in which this adaptive mesh definition is active. The default value is 5.
    initialMeshSweeps: int = 5

    #: An Int specifying the number of mesh sweeps to be performed in each adaptive mesh
    #: increment. The default value is 1.
    meshSweeps: int = 1

    @abaqus_method_doc
    def __init__(
        self,
        region: Region,
        controls: str = "",
        frequency: int = 10,
        initialMeshSweeps: int = 5,
        meshSweeps: int = 1,
    ):
        """This method creates an AdaptiveMeshDomain object.

        .. note::
            This function can be accessed by::

                mdb.models[name].steps[name].AdaptiveMeshDomain

        Parameters
        ----------
        region
            A Region object specifying the region to which the adaptive mesh domain is applied.
        controls
            A String specifying the name of an AdaptiveMeshControl object.
        frequency
            An Int specifying the frequency in increments at which adaptive meshing will be
            performed. The default value is 10.
        initialMeshSweeps
            An Int specifying the number of mesh sweeps to be performed at the beginning of the
            first step in which this adaptive mesh definition is active. The default value is 5.
        meshSweeps
            An Int specifying the number of mesh sweeps to be performed in each adaptive mesh
            increment. The default value is 1.

        Returns
        -------
        AdaptiveMeshDomain
            An AdaptiveMeshDomain object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the AdaptiveMeshDomain object.

        Raises
        ------
        RangeError
        """
        ...
